# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

import json
from typing import Optional, List

from kag.common.base.prompt_op import PromptOp


class OpenIEEntitystandardizationdPrompt(PromptOp):

    template_zh = """
{
    "instruction": "input字段包含用户提供的上下文。命名实体字段包含从上下文中提取的命名实体，这些可能是含义不明的缩写、别名或俚语。为了消除歧义，请尝试根据上下文和您自己的知识提供这些实体的官方名称。请注意，具有相同含义的实体只能有一个官方名称。请按照提供的示例中的输出字段格式，以单个JSONArray字符串形式回复，无需任何解释。",
    "example": {
        "input": "烦躁不安、语妄、失眠酌用镇静药，禁用抑制呼吸的镇静药。\n3.并发症的处理经抗菌药物治疗后，高热常在24小时内消退，或数日内逐渐下降。\n若体温降而复升或3天后仍不降者，应考虑SP的肺外感染，如腋胸、心包炎或关节炎等。治疗：接胸腔压力调节管＋吸引机负压吸引水瓶装置闭式负压吸引宜连续，如经12小时后肺仍未复张，应查找原因。",
        "named_entities": [
            {"entity": "烦躁不安", "category": "Symptom"},
            {"entity": "语妄", "category": "Symptom"},
            {"entity": "失眠", "category": "Symptom"},
            {"entity": "镇静药", "category": "Medicine"},
            {"entity": "肺外感染", "category": "Disease"},
            {"entity": "胸腔压力调节管", "category": "MedicalEquipment"},
            {"entity": "吸引机负压吸引水瓶装置", "category": "MedicalEquipment"},
            {"entity": "闭式负压吸引", "category": "SurgicalOperation"}
        ],
        "output": [
            {"entity": "烦躁不安", "category": "Symptom", "official_name": "焦虑不安"},
            {"entity": "语妄", "category": "Symptom", "official_name": "谵妄"},
            {"entity": "失眠", "category": "Symptom", "official_name": "失眠症"},
            {"entity": "镇静药", "category": "Medicine", "official_name": "镇静剂"},
            {"entity": "肺外感染", "category": "Disease", "official_name": "肺外感染"},
            {"entity": "胸腔压力调节管", "category": "MedicalEquipment", "official_name": "胸腔引流管"},
            {"entity": "吸引机负压吸引水瓶装置", "category": "MedicalEquipment", "official_name": "负压吸引装置"},
            {"entity": "闭式负压吸引", "category": "SurgicalOperation", "official_name": "闭式负压引流"}
        ]
    },
    "input": $input,
    "named_entities": $named_entities,
}    
    """

    template_en = template_zh

    def __init__(self, language: Optional[str] = "en"):
        super().__init__(language)

    @property
    def template_variables(self) -> List[str]:
        return ["input", "named_entities"]

    def parse_response(self, response: str, **kwargs):

        rsp = response
        if isinstance(rsp, str):
            rsp = json.loads(rsp)
        if isinstance(rsp, dict) and "output" in rsp:
            rsp = rsp["output"]
        if isinstance(rsp, dict) and "named_entities" in rsp:
            standardized_entity = rsp["named_entities"]
        else:
            standardized_entity = rsp
        entities_with_offical_name = set()
        merged = []
        entities = kwargs.get("named_entities", [])
        for entity in standardized_entity:
            merged.append(entity)
            entities_with_offical_name.add(entity["entity"])
        # in case llm ignores some entities
        for entity in entities:
            if entity["entity"] not in entities_with_offical_name:
                entity["official_name"] = entity["entity"]
                merged.append(entity)
        return merged



#!/usr/bin/python
# coding=utf8
import concurrent.futures
import logging
from typing import List, Union

from kag.interface.retriever.kg_retriever_abc import KGRetrieverABC
from kag.solver.logic.core_modules.common.base_model import SPOEntity
from kag.solver.logic.core_modules.common.one_hop_graph import KgGraph, EntityData
from kag.solver.logic.core_modules.common.schema_utils import SchemaUtils
from kag.solver.logic.core_modules.common.text_sim_by_vector import TextSimilarity
from kag.solver.logic.core_modules.parser.logic_node_parser import GetSPONode, GetNode

logger = logging.getLogger()


class EntityLinkerBase:
    def __init__(self, config):
        self.config = config

    def entity_linking(self, content, entities: List[SPOEntity], req_id='', **kwargs):
        logger.info(f"EntityLinkerBase {req_id} return empty linker")
        return [
               ], []

    def get_service_name(self):
        return {
            'scene_name': '空链指调用'
        }


class DefaultEntityLinker(EntityLinkerBase):
    def __init__(self, config, kg_retriever: KGRetrieverABC):
        super().__init__(config)
        self.recognition_threshold = float(0.8)
        self.kg_retriever = kg_retriever

    def get_service_name(self):
        return {
            'scene_name': 'neo4j'
        }

    def _call_feature(self, feature):
        mention_entity = feature.get('mention_entity', None)
        return self.kg_retriever.retrieval_entity(mention_entity, params=feature)

    def compose_features(self, content, entities: List[SPOEntity], req_id='', params={}):
        features = []
        for i, entity in enumerate(entities):
            content = f"{content}[Entity]{entity.entity_name}"
            feature = {
                "mention_entity": entity,
                "property_key": "name",
                'content': content,
                "query_text": entity.entity_name,
                'recognition_threshold': self.recognition_threshold
            }
            feature.update(params)
            features.append(feature)
        return features

    ## ha3召回+精排链指
    def entity_linking(self, content, entities: List[SPOEntity], req_id='', **kwargs):
        '''
        input:
            content: str, context
            entities: [], entity spans to be linked
            types: [], entity types to be linked
        output:
            [{'content': '吉林省抚松县被人们称为是哪种药材之乡？', 'entities': [{'word': '吉林省抚松县', 'start_idx': 0, 'recall': []}]}
        '''
        features = self.compose_features(content, entities, req_id, kwargs)
        entity_recalls = {}
        logger.debug(f"{req_id} entity_linking {features}")
        call_datas = []
        if len(features) == 1:
            res = self._call_feature(features[0])
            call_datas.append({'res': res, 'recalls': entity_recalls, 'content': content})
        else:
            with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
                call_datas = [{'res': d, 'recalls': entity_recalls, 'content': content} for d in
                              list(executor.map(self._call_feature, features))]
        logger.debug(f'{req_id} entity_linking result: {call_datas}')
        results = []
        for data in call_datas:
            recalled_entities = data['res']
            results.append(recalled_entities)
        return results, call_datas


def spo_entity_linker(kg_graph: KgGraph, n: Union[GetSPONode, GetNode], nl_query, el: EntityLinkerBase, schema: SchemaUtils, req_id='',
                      params={}):
    el_results = []
    call_result_data = []
    entities_candis = []
    args_entity_mentions = [[], [], []]  # [keys, entities_name, entities_type]
    s_data = kg_graph.get_entity_by_alias(n.s.alias_name)
    if s_data is None and isinstance(n.s, SPOEntity) and n.s.entity_name and len(n.s.id_set) == 0:
        entities_candis.append(n.s)

    el_kg_graph = KgGraph()
    if isinstance(n, GetSPONode):
        o_data = kg_graph.get_entity_by_alias(n.o.alias_name)
        if o_data is None and isinstance(n.o, SPOEntity) and n.o.entity_name and len(n.o.id_set) == 0:
            entities_candis.append(n.o)
        el_kg_graph.query_graph[n.p.alias_name] = {
            "s": n.s.alias_name,
            "p": n.p.alias_name,
            "o": n.o.alias_name
        }

    el_request = {
        "nl_query": nl_query,
        "entity_mentions": entities_candis
    }
    err_msg = ""
    if entities_candis and el is not None:
        try:
            el_results, call_result_data = el.entity_linking(nl_query, entities_candis, req_id, kwargs=params)
        except Exception as e:
            logger.error(f"{req_id} spo_entity_linker error, we need use name to id {str(e)}", exc_info=True)
            el_results = []
            call_result_data = []
            err_msg = str(e)
        for i in range(len(entities_candis)):
            candis_entitiy = entities_candis[i]
            entity_data_set = []
            if el_results and i < len(el_results) and el_results[i] is not None and len(el_results[i]) > 0:
                el_recalls = el_results[i]
                for entity_id_info in el_recalls:
                    entity_type_zh = schema.node_en_zh[
                        entity_id_info.type] if schema is not None and entity_id_info.type in schema.node_en_zh.keys() else None
                    entity_id_info.type_zh = entity_type_zh
                    entity_data_set.append(entity_id_info)
            else:
                entity_id_info = EntityData()
                entity_id_info.name = candis_entitiy.entity_name
                entity_id_info.biz_id = candis_entitiy.entity_name
                entity_id_info.type = schema.get_label_within_prefix(candis_entitiy.get_entity_first_type())
                entity_type_zh = schema.node_en_zh[
                    entity_id_info.type] if schema is not None and entity_id_info.type in schema.node_en_zh.keys() else None
                entity_id_info.type_zh = entity_type_zh
                entity_data_set.append(entity_id_info)
            el_kg_graph.nodes_alias.append(candis_entitiy.alias_name)
            el_kg_graph.entity_map[candis_entitiy.alias_name] = entity_data_set

    kg_graph.merge_kg_graph(el_kg_graph, True)
    return el_results, el_request, err_msg, call_result_data



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.
import copy
import logging
import os
from typing import Dict, Type, List

from tenacity import stop_after_attempt, retry

from kag.builder.prompt.spg_prompt import SPG_KGPrompt
from kag.interface.builder import ExtractorABC
from kag.common.base.prompt_op import PromptOp
from knext.schema.client import OTHER_TYPE, CHUNK_TYPE, BASIC_TYPES
from kag.common.utils import processing_phrases, to_camel_case
from kag.builder.model.chunk import Chunk
from kag.builder.model.sub_graph import SubGraph
from knext.common.base.runnable import Input, Output
from knext.schema.client import SchemaClient
from knext.schema.model.base import SpgTypeEnum

logger = logging.getLogger(__name__)


class KAGExtractor(ExtractorABC):
    """
    A class for extracting knowledge graph subgraphs from text using a large language model (LLM).
    Inherits from the Extractor base class.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.llm = self._init_llm()
        self.prompt_config = self.config.get("prompt", {})
        self.biz_scene = self.prompt_config.get("biz_scene") or os.getenv("KAG_PROMPT_BIZ_SCENE", "default")
        self.language = self.prompt_config.get("language") or os.getenv("KAG_PROMPT_LANGUAGE", "en")
        self.schema = SchemaClient(project_id=self.project_id).load()
        self.ner_prompt = PromptOp.load(self.biz_scene, "ner")(language=self.language, project_id=self.project_id)
        self.std_prompt = PromptOp.load(self.biz_scene, "std")(language=self.language)
        self.triple_prompt = PromptOp.load(self.biz_scene, "triple")(
            language=self.language
        )
        self.kg_types = []
        for type_name, spg_type in self.schema.items():
            if type_name in SPG_KGPrompt.ignored_types:
                continue
            if spg_type.spg_type_enum == SpgTypeEnum.Concept:
                continue
            properties = list(spg_type.properties.keys())
            for p in properties:
                if p not in SPG_KGPrompt.ignored_properties:
                    self.kg_types.append(type_name)
                    break
        if self.kg_types:
            self.kg_prompt = SPG_KGPrompt(self.kg_types, language=self.language, project_id=self.project_id)

    @property
    def input_types(self) -> Type[Input]:
        return Chunk

    @property
    def output_types(self) -> Type[Output]:
        return SubGraph

    @retry(stop=stop_after_attempt(3))
    def named_entity_recognition(self, passage: str):
        """
        Performs named entity recognition on a given text passage.
        Args:
            passage (str): The text to perform named entity recognition on.
        Returns:
            The result of the named entity recognition operation.
        """
        if self.kg_types:
            kg_result = self.llm.invoke({"input": passage}, self.kg_prompt)
        else:
            kg_result = []
        ner_result = self.llm.invoke({"input": passage}, self.ner_prompt)
        return kg_result + ner_result

    @retry(stop=stop_after_attempt(3))
    def named_entity_standardization(self, passage: str, entities: List[Dict]):
        """
        Standardizes named entities.

        Args:
            passage (str): The input text passage.
            entities (List[Dict]): A list of recognized named entities.

        Returns:
            Standardized entity information.
        """
        return self.llm.invoke(
            {"input": passage, "named_entities": entities}, self.std_prompt
        )

    @retry(stop=stop_after_attempt(3))
    def triples_extraction(self, passage: str, entities: List[Dict]):
        """
        Extracts triples (subject-predicate-object structures) from a given text passage based on identified entities.
        Args:
            passage (str): The text to extract triples from.
            entities (List[Dict]): A list of entities identified in the text.
        Returns:
            The result of the triples extraction operation.
        """
        return self.llm.invoke(
            {"input": passage, "entity_list": entities}, self.triple_prompt
        )

    def assemble_sub_graph_with_spg_records(self, entities: List[Dict]):
        sub_graph = SubGraph([], [])
        for record in entities:
            s_name = record.get("entity", "")
            s_label = record.get("category", "")
            properties = record.get("properties", {})
            tmp_properties = copy.deepcopy(properties)
            spg_type = self.schema.get(s_label)
            for prop_name, prop_value in properties.items():
                if prop_value == "NAN":
                    tmp_properties.pop(prop_name)
                    continue
                if prop_name in spg_type.properties:
                    from knext.schema.model.property import Property
                    prop: Property = spg_type.properties.get(prop_name)
                    o_label = prop.object_type_name_en
                    if o_label not in BASIC_TYPES:
                        if isinstance(prop_value, str):
                            prop_value = [prop_value]
                        for o_name in prop_value:
                            sub_graph.add_node(id=o_name, name=o_name, label=o_label)
                            sub_graph.add_edge(s_id=s_name, s_label=s_label, p=prop_name, o_id=o_name, o_label=o_label)
                        tmp_properties.pop(prop_name)
            record["properties"] = tmp_properties
            sub_graph.add_node(id=s_name, name=s_name, label=s_label, properties=properties)
        return sub_graph, entities

    @staticmethod
    def assemble_sub_graph_with_triples(
        sub_graph: SubGraph, entities: List[Dict], triples: List[list]
    ):
        """
        Assembles edges in the subgraph based on a list of triples and entities.
        Args:
            sub_graph (SubGraph): The subgraph to add edges to.
            entities (List[Dict]): A list of entities, for looking up category information.
            triples (List[list]): A list of triples, each representing a relationship to be added to the subgraph.
        """

        def get_category(entities_data, entity_name):
            for entity in entities_data:
                if entity["entity"] == entity_name:
                    return entity["category"]
            return None

        for tri in triples:
            if len(tri) != 3:
                continue
            s_category = get_category(entities, tri[0])
            tri[0] = processing_phrases(tri[0])
            if s_category is None:
                s_category = OTHER_TYPE
                sub_graph.add_node(tri[0], tri[0], s_category)
            o_category = get_category(entities, tri[2])
            tri[2] = processing_phrases(tri[2])
            if o_category is None:
                o_category = OTHER_TYPE
                sub_graph.add_node(tri[2], tri[2], o_category)

            sub_graph.add_edge(
                tri[0], s_category, to_camel_case(tri[1]), tri[2], o_category
            )

        return sub_graph

    @staticmethod
    def assemble_sub_graph_with_chunk(sub_graph: SubGraph, chunk: Chunk):
        """
        Associates a Chunk object with the subgraph, adding it as a node and connecting it with existing nodes.
        Args:
            sub_graph (SubGraph): The subgraph to add the chunk information to.
            chunk (Chunk): The chunk object containing the text and metadata.
        """
        for node in sub_graph.nodes:
            sub_graph.add_edge(node.id, node.label, "source", chunk.id, CHUNK_TYPE)
        sub_graph.add_node(
            chunk.id,
            chunk.name,
            CHUNK_TYPE,
            {
                "id": chunk.id,
                "name": chunk.name,
                "content": f"{chunk.name}\n{chunk.content}",
                **chunk.kwargs
            },
        )
        sub_graph.id = chunk.id
        return sub_graph

    def assemble_sub_graph(
        self, sub_graph: SubGraph, chunk: Chunk, entities: List[Dict], triples: List[list]
    ):
        """
        Integrates entity and triple information into a subgraph, and associates it with a chunk of text.
        Args:
            sub_graph (SubGraph): The subgraph to be assembled.
            chunk (Chunk): The chunk of text the subgraph is about.
            entities (List[Dict]): A list of entities identified in the chunk.
            triples (List[list]): A list of triples representing relationships between entities.
        Returns:
            SubGraph: The constructed subgraph.
        """
        self.assemble_sub_graph_with_entities(sub_graph, entities)
        self.assemble_sub_graph_with_triples(sub_graph, entities, triples)
        self.assemble_sub_graph_with_chunk(sub_graph, chunk)
        return sub_graph

    def assemble_sub_graph_with_entities(
        self, sub_graph: SubGraph, entities: List[Dict]
    ):
        """
        Assembles a subgraph using named entities.

        Args:
            sub_graph (SubGraph): The subgraph object to be assembled.
            entities (List[Dict]): A list containing entity information.
        """

        for ent in entities:
            name = processing_phrases(ent["entity"])
            sub_graph.add_node(
                name,
                name,
                ent["category"],
                {
                    "desc": ent.get("description", ""),
                    "semanticType": ent.get("type", ""),
                    **ent.get("properties", {}),
                },
            )

            if "official_name" in ent:
                official_name = processing_phrases(ent["official_name"])
                if official_name != name:
                    sub_graph.add_node(
                        official_name,
                        official_name,
                        ent["category"],
                        {
                            "desc": ent.get("description", ""),
                            "semanticType": ent.get("type", ""),
                            **ent.get("properties", {}),
                        },
                    )
                    sub_graph.add_edge(
                        name,
                        ent["category"],
                        "OfficialName",
                        official_name,
                        ent["category"],
                    )

    def append_official_name(
        self, source_entities: List[Dict], entities_with_official_name: List[Dict]
    ):
        """
        Appends official names to entities.

        Args:
            source_entities (List[Dict]): A list of source entities.
            entities_with_official_name (List[Dict]): A list of entities with official names.
        """
        tmp_dict = {}
        for tmp_entity in entities_with_official_name:
            name = tmp_entity["entity"]
            category = tmp_entity["category"]
            official_name = tmp_entity["official_name"]
            key = f"{category}{name}"
            tmp_dict[key] = official_name

        for tmp_entity in source_entities:
            name = tmp_entity["entity"]
            category = tmp_entity["category"]
            key = f"{category}{name}"
            if key in tmp_dict:
                official_name = tmp_dict[key]
                tmp_entity["official_name"] = official_name

    def invoke(self, input: Input, **kwargs) -> List[Output]:
        """
        Invokes the semantic extractor to process input data.

        Args:
            input (Input): Input data containing name and content.
            **kwargs: Additional keyword arguments.

        Returns:
            List[Output]: A list of processed results, containing subgraph information.
        """
        title = input.name
        passage = title + "\n" + input.content

        try:
            entities = self.named_entity_recognition(passage)
            sub_graph, entities = self.assemble_sub_graph_with_spg_records(entities)
            filtered_entities = [{k: v for k, v in ent.items() if k in ["entity", "category"]} for ent in entities]
            triples = self.triples_extraction(passage, filtered_entities)
            std_entities = self.named_entity_standardization(passage, filtered_entities)
            self.append_official_name(entities, std_entities)
            self.assemble_sub_graph(sub_graph, input, entities, triples)
            return [sub_graph]
        except Exception as e:
            import traceback

            traceback.print_exc()
            logger.info(e)
        return []



import json
import logging
import os
import time
from typing import List

from kag.common.vectorizer import Vectorizer
from kag.interface.retriever.chunk_retriever_abc import ChunkRetrieverABC
from kag.interface.retriever.kg_retriever_abc import KGRetrieverABC
from kag.solver.logic.core_modules.common.base_model import LFPlanResult
from kag.solver.logic.core_modules.common.schema_utils import SchemaUtils
from kag.solver.logic.core_modules.common.text_sim_by_vector import TextSimilarity
from kag.solver.logic.core_modules.common.utils import generate_random_string
from kag.solver.logic.core_modules.config import LogicFormConfiguration
from kag.solver.logic.core_modules.lf_executor import LogicExecutor
from kag.solver.logic.core_modules.lf_generator import LFGenerator
from kag.solver.logic.core_modules.retriver.entity_linker import DefaultEntityLinker
from kag.solver.logic.core_modules.retriver.graph_retriver.dsl_executor import DslRunnerOnGraphStore
from kag.solver.logic.core_modules.retriver.schema_std import SchemaRetrieval
from knext.project.client import ProjectClient

logger = logging.getLogger()


class LFSolver:
    """
    Solver class that integrates various components to solve queries using logic forms.
    This class can't be extended to implement custom solver strategies.
    """

    def __init__(self, kg_retriever: KGRetrieverABC = None,
                 chunk_retriever: ChunkRetrieverABC = None,
                 report_tool=None, **kwargs):
        """
        Initializes the solver with necessary modules and configurations.

        Parameters:
        chunk_retriever (ChunkRetriever): An instance for chunk-level retrieval. If not provided, we will not execute chunk retrieval.
        kg_retriever (KGRetrieval): An instance for graph-level retrieval. If not provided, we will not execute retrieval on graph.
        report_tool (Tool, optional): An instance of the reporting tool. Defaults to None.

        Returns:
        None

        Raises:
        ValueError: If both `kg_retriever` and `chunk_retriever` are None.
        """
        if kg_retriever is None and chunk_retriever is None:
            raise ValueError("At least one of `kg_retriever` or `chunk_retriever` must be provided.")

        self.kg_retriever = kg_retriever
        self.chunk_retriever = chunk_retriever
        self.project_id = kwargs.get("KAG_PROJECT_ID") or os.getenv("KAG_PROJECT_ID")
        self.host_addr = kwargs.get("KAG_PROJECT_HOST_ADDR") or os.getenv("KAG_PROJECT_HOST_ADDR")
        if report_tool and report_tool.project_id:
            self.project_id = report_tool.project_id

        self.schema = SchemaUtils(LogicFormConfiguration(kwargs))
        self.schema.get_schema()
        self.std_schema = SchemaRetrieval(**kwargs)
        self.el = DefaultEntityLinker(None, self.kg_retriever)
        self.generator = LFGenerator(**kwargs)
        self.report_tool = report_tool
        self.last_iter_docs = []

        vectorizer_config = eval(os.getenv("KAG_VECTORIZER", "{}"))
        if self.host_addr and self.project_id:
            config = ProjectClient(host_addr=self.host_addr, project_id=self.project_id).get_config(self.project_id)
            vectorizer_config.update(config.get("vectorizer", {}))
        self.vectorizer: Vectorizer = Vectorizer.from_config(vectorizer_config)
        self.text_similarity = TextSimilarity(vec_config=vectorizer_config)

    def _process_history(self, history):
        """
        Processes the history to extract sub-query-answer pairs and document sets.

        Parameters:
        history (list): A list of historical query-answer pairs.

        Returns:
        tuple: A tuple containing the list of sub-query-answer pairs and the list of document sets.
        """
        sub_qa_pair = []
        docs_set = []
        for i, h in enumerate(history):
            if "sub_query" not in h:
                continue
            if 'sub_answer' in h and h['sub_answer'].lower() != "i don't know":
                sub_qa_pair.append(f"query{i + 1}: {h['sub_query']}\nanswer{i + 1}: {h['sub_answer']}")
            if "docs" in h and len(h['docs']) > 0:
                docs_set.append(h['docs'])
        return sub_qa_pair, docs_set

    def _flat_passages_set(self, passages_set: list):
        """
        Flattens the passages set and scores each passage based on its position.

        Parameters:
        passages_set (list): A list of passage sets.

        Returns:
        list: A list of passages sorted by their scores.
        """
        score_map = {}
        if len(self.last_iter_docs) > 0:
            passages_set.append(self.last_iter_docs)
        for passages in passages_set:
            passages = ["#".join(item.split("#")[:-1]) for item in passages]
            for i, passage in enumerate(passages):
                score = 1.0 / (1 + i)
                if passage in score_map:
                    score_map[passage] += score
                else:
                    score_map[passage] = score

        return [k for k, v in sorted(score_map.items(), key=lambda item: item[1], reverse=True)]

    def solve(self, query, lf_nodes: List[LFPlanResult]):
        """
        Solves the query using logic forms and returns the results.

        Parameters:
        query (str): The main query to solve.
        lf_nodes (list): A list of LFPlanResult to be solved.

        Returns:
        tuple: A tuple containing the final answer, sub-query-answer pairs, relevant documents, and history.
        """
        try:
            start_time = time.time()
            executor = LogicExecutor(
                query, self.project_id, self.schema,
                kg_retriever=self.kg_retriever,
                chunk_retriever=self.chunk_retriever,
                std_schema=self.std_schema,
                el=self.el,
                text_similarity=self.text_similarity,
                dsl_runner=DslRunnerOnGraphStore(self.project_id, self.schema, LogicFormConfiguration({
                    "KAG_PROJECT_ID": self.project_id,
                    "KAG_PROJECT_HOST_ADDR": self.host_addr
                })),
                generator=self.generator,
                report_tool=self.report_tool,
                req_id=generate_random_string(10)
            )
            kg_qa_result, kg_graph, history = executor.execute(lf_nodes, query)
            logger.info(
                f"{executor.req_id} call_kb_paths cost={time.time() - start_time} kg_path={kg_graph.to_answer_path()}"
            )
        except Exception as e:
            logger.warning(f"lf_retriever {query} lf failed {str(e)}", exc_info=True)
            history = []
            kg_qa_result = []

        docs = []
        sub_qa_pair = []
        if history:
            sub_qa_pair, docs_set = self._process_history(history)
            docs = self._flat_passages_set(docs_set)
        if len(docs) == 0 and len(sub_qa_pair) == 0 and self.chunk_retriever:
            cur_step_recall_docs = self.chunk_retriever.recall_docs(query)
            history.append({'docs': cur_step_recall_docs})
            docs = self._flat_passages_set([cur_step_recall_docs])
        if len(docs) != 0:
            self.last_iter_docs = docs
        return ",".join(kg_qa_result), sub_qa_pair, docs, history



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

import json
from string import Template
from typing import List, Optional

from kag.common.base.prompt_op import PromptOp
from knext.schema.client import SchemaClient


class OpenIENERPrompt(PromptOp):

    template_zh = """
    {
        "instruction": "你是命名实体识别的专家。请从输入中提取与模式定义匹配的实体。如果不存在该类型的实体，请返回一个空列表。请以JSON字符串格式回应。你可以参照example进行抽取。",
        "schema": $schema,
        "example": [
            {
                "input": "烦躁不安、语妄、失眠酌用镇静药，禁用抑制呼吸的镇静药。\n3.并发症的处理经抗菌药物治疗后，高热常在24小时内消退，或数日内逐渐下降。\n若体温降而复升或3天后仍不降者，应考虑SP的肺外感染。\n治疗：接胸腔压力调节管＋吸引机负压吸引水瓶装置闭式负压吸引宜连续，如经12小时后肺仍未复张，应查找原因。",
                "output": [
                        {"entity": "烦躁不安", "category": "Symptom"},
                        {"entity": "语妄", "category": "Symptom"},
                        {"entity": "失眠", "category": "Symptom"},
                        {"entity": "镇静药", "category": "Medicine"},
                        {"entity": "肺外感染", "category": "Disease"},
                        {"entity": "胸腔压力调节管", "category": "MedicalEquipment"},
                        {"entity": "吸引机负压吸引水瓶装置", "category": "MedicalEquipment"},
                        {"entity": "闭式负压吸引", "category": "SurgicalOperation"}
                    ]
            }
        ],
        "input": "$input"
    }    
        """

    template_en = template_zh

    def __init__(
            self, language: Optional[str] = "en", **kwargs
    ):
        super().__init__(language, **kwargs)
        self.schema = SchemaClient(project_id=self.project_id).extract_types()
        self.template = Template(self.template).safe_substitute(schema=self.schema)

    @property
    def template_variables(self) -> List[str]:
        return ["input"]

    def parse_response(self, response: str, **kwargs):
        rsp = response
        if isinstance(rsp, str):
            rsp = json.loads(rsp)
        if isinstance(rsp, dict) and "output" in rsp:
            rsp = rsp["output"]
        if isinstance(rsp, dict) and "named_entities" in rsp:
            entities = rsp["named_entities"]
        else:
            entities = rsp

        return entities



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

from kag.common.vectorizer.local_bge_m3_vectorizer import LocalBGEM3Vectorizer
from kag.common.vectorizer.local_bge_vectorizer import LocalBGEVectorizer
from kag.common.vectorizer.openai_vectorizer import OpenAIVectorizer
from kag.common.vectorizer.vectorizer import Vectorizer
from kag.common.vectorizer.vectorizer_config_checker import VectorizerConfigChecker


__all__ = [
    "LocalBGEM3Vectorizer",
    "LocalBGEVectorizer",
    "OpenAIVectorizer",
    "Vectorizer",
    "VectorizerConfigChecker",
]



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

from typing import List, Sequence, Dict, Type

from kag.builder.model.sub_graph import SubGraph
from kag.interface.builder import AlignerABC
from knext.common.base.runnable import Input, Output


class KAGPostProcessorAligner(AlignerABC):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def input_types(self) -> Type[Input]:
        return SubGraph

    @property
    def output_types(self) -> Type[Output]:
        return SubGraph

    def invoke(self, input: List[SubGraph], **kwargs) -> SubGraph:
        merged_sub_graph = SubGraph(nodes=[], edges=[])
        for sub_graph in input:
            for node in sub_graph.nodes:
                if node not in merged_sub_graph.nodes:
                    merged_sub_graph.nodes.append(node)
            for edge in sub_graph.edges:
                if edge not in merged_sub_graph.edges:
                    merged_sub_graph.edges.append(edge)
        return merged_sub_graph

    def _handle(self, input: Sequence[Dict]) -> Dict:
        _input = [self.input_types.from_dict(i) for i in input]
        _output = self.invoke(_input)
        return _output.to_dict()

    def batch(self, inputs: List[Input], **kwargs) -> List[Output]:
        pass



from tenacity import retry, stop_after_attempt

from kag.common.base.prompt_op import PromptOp
from kag.interface.solver.kag_reflector_abc import KagMemoryABC
from kag.interface.solver.kag_reflector_abc import KagReflectorABC


class DefaultReflector(KagReflectorABC):
    def __init__(self, **kwargs):
        """
        A class for rewriting instructions based on provided memory information.

        Attributes:
        - llm_module (Any): The LLM module to be used by this instance.
        - rewrite_prompt (PromptOp): The prompt operation for rewriting responses.
        """
        super().__init__(**kwargs)
        self.refine_prompt = PromptOp.load(self.biz_scene, "resp_reflector")(
            language=self.language
        )

        self.judge_prompt = PromptOp.load(self.biz_scene, "resp_judge")(
            language=self.language
        )

    def _get_serialize_memory(self, memory: KagMemoryABC):
        if memory is None:
            return ""
        return memory.serialize_memory()

    @retry(stop=stop_after_attempt(3))
    def _can_answer(self, memory: KagMemoryABC, instruction: str):
        """
        Determines whether the query can be answered.

        :param memory (KagMemory): The context or memory information to use for rewriting.
        :param instruction (str): The original instruction to be rewritten.
        :return: Whether the query can be answered (boolean)
        """
        serialize_memory = self._get_serialize_memory(memory)
        if serialize_memory == "":
            return False

        if memory.get_solved_answer() != "":
            return True

        return self.llm_module.invoke({'memory': serialize_memory, 'instruction': instruction}, self.judge_prompt,
                                      with_json_parse=False, with_except=True)

    @retry(stop=stop_after_attempt(3))
    def _refine_query(self, memory: KagMemoryABC, instruction: str):
        """
        Refines the query.

        :param memory (KagMemory): The context or memory information to use for rewriting.
        :param instruction (str): The original instruction to be rewritten.
        :return: The refined query (string)
        """
        serialize_memory = self._get_serialize_memory(memory)
        if serialize_memory == "":
            return instruction

        update_reason_path = self.llm_module.invoke({"memory": serialize_memory, "instruction": instruction},
                                                    self.refine_prompt,
                                                    with_json_parse=False, with_except=True)
        if len(update_reason_path) == 0:
            return None
        return update_reason_path[0]


# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.
import logging
import os
from enum import Enum
from typing import Type, Dict, List

from knext.graph_algo.client import GraphAlgoClient
from kag.builder.model.sub_graph import SubGraph
from kag.interface.builder.writer_abc import SinkWriterABC
from knext.common.base.runnable import Input, Output

logger = logging.getLogger(__name__)


class AlterOperationEnum(str, Enum):
    Upsert = "UPSERT"
    Delete = "DELETE"


class KGWriter(SinkWriterABC):
    """
    A class that extends `SinkWriter` to handle writing data into a Neo4j knowledge graph.

    This class is responsible for configuring the graph store based on environment variables and
    an optional project ID, initializing the Neo4j client, and setting up the schema.
    It also manages semantic indexing and multi-threaded operations.
    """

    def __init__(self, project_id: str = None, **kwargs):
        super().__init__(**kwargs)
        self.project_id = project_id or os.getenv("KAG_PROJECT_ID")
        self.client = GraphAlgoClient(project_id=project_id)

    @property
    def input_types(self) -> Type[Input]:
        return SubGraph

    @property
    def output_types(self) -> Type[Output]:
        return None

    def invoke(
        self, input: Input, alter_operation: str = AlterOperationEnum.Upsert, lead_to_builder: bool = False
    ) -> List[Output]:
        """
        Invokes the specified operation (upsert or delete) on the graph store.

        Args:
            input (Input): The input object representing the subgraph to operate on.
            alter_operation (str): The type of operation to perform (Upsert or Delete).
            lead_to_builder (str): enable lead to event infer builder

        Returns:
            List[Output]: A list of output objects (currently always [None]).
        """
        self.client.write_graph(sub_graph=input.to_dict(), operation=alter_operation, lead_to_builder=lead_to_builder)
        return [None]

    def _handle(self, input: Dict, alter_operation: str, **kwargs):
        """The calling interface provided for SPGServer."""
        _input = self.input_types.from_dict(input)
        _output = self.invoke(_input, alter_operation)
        return None



from typing import Union

from kag.solver.logic.core_modules.common.base_model import LogicNode
from kag.solver.logic.core_modules.common.one_hop_graph import KgGraph
from kag.solver.logic.core_modules.common.schema_utils import SchemaUtils
from kag.solver.logic.core_modules.op_executor.op_executor import OpExecutor
from kag.solver.logic.core_modules.parser.logic_node_parser import SortNode


class SortExecutor(OpExecutor):
    def __init__(self, nl_query: str, kg_graph: KgGraph, schema: SchemaUtils, debug_info: dict, **kwargs):
        super().__init__(nl_query, kg_graph, schema, debug_info, **kwargs)

    def is_this_op(self, logic_node: LogicNode) -> bool:
        return isinstance(logic_node, SortNode)
    def executor(self, logic_node: LogicNode, req_id: str, param: dict) -> Union[KgGraph, list]:
        pass



# coding: utf-8

"""
    kag

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kag.common.rest.configuration import Configuration


class EdgeRecordInstance(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'src_type': 'str',
        'src_id': 'str',
        'dst_type': 'str',
        'dst_id': 'str',
        'label': 'str',
        'properties': 'object'
    }

    attribute_map = {
        'src_type': 'srcType',
        'src_id': 'srcId',
        'dst_type': 'dstType',
        'dst_id': 'dstId',
        'label': 'label',
        'properties': 'properties'
    }

    def __init__(self, src_type=None, src_id=None, dst_type=None, dst_id=None, label=None, properties=None, local_vars_configuration=None):  # noqa: E501
        """EdgeRecordInstance - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._src_type = None
        self._src_id = None
        self._dst_type = None
        self._dst_id = None
        self._label = None
        self._properties = None
        self.discriminator = None

        self.src_type = src_type
        self.src_id = src_id
        self.dst_type = dst_type
        self.dst_id = dst_id
        self.label = label
        self.properties = properties

    @property
    def src_type(self):
        """Gets the src_type of this EdgeRecordInstance.  # noqa: E501


        :return: The src_type of this EdgeRecordInstance.  # noqa: E501
        :rtype: str
        """
        return self._src_type

    @src_type.setter
    def src_type(self, src_type):
        """Sets the src_type of this EdgeRecordInstance.


        :param src_type: The src_type of this EdgeRecordInstance.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and src_type is None:  # noqa: E501
            raise ValueError("Invalid value for `src_type`, must not be `None`")  # noqa: E501

        self._src_type = src_type

    @property
    def src_id(self):
        """Gets the src_id of this EdgeRecordInstance.  # noqa: E501


        :return: The src_id of this EdgeRecordInstance.  # noqa: E501
        :rtype: str
        """
        return self._src_id

    @src_id.setter
    def src_id(self, src_id):
        """Sets the src_id of this EdgeRecordInstance.


        :param src_id: The src_id of this EdgeRecordInstance.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and src_id is None:  # noqa: E501
            raise ValueError("Invalid value for `src_id`, must not be `None`")  # noqa: E501

        self._src_id = src_id

    @property
    def dst_type(self):
        """Gets the dst_type of this EdgeRecordInstance.  # noqa: E501


        :return: The dst_type of this EdgeRecordInstance.  # noqa: E501
        :rtype: str
        """
        return self._dst_type

    @dst_type.setter
    def dst_type(self, dst_type):
        """Sets the dst_type of this EdgeRecordInstance.


        :param dst_type: The dst_type of this EdgeRecordInstance.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and dst_type is None:  # noqa: E501
            raise ValueError("Invalid value for `dst_type`, must not be `None`")  # noqa: E501

        self._dst_type = dst_type

    @property
    def dst_id(self):
        """Gets the dst_id of this EdgeRecordInstance.  # noqa: E501


        :return: The dst_id of this EdgeRecordInstance.  # noqa: E501
        :rtype: str
        """
        return self._dst_id

    @dst_id.setter
    def dst_id(self, dst_id):
        """Sets the dst_id of this EdgeRecordInstance.


        :param dst_id: The dst_id of this EdgeRecordInstance.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and dst_id is None:  # noqa: E501
            raise ValueError("Invalid value for `dst_id`, must not be `None`")  # noqa: E501

        self._dst_id = dst_id

    @property
    def label(self):
        """Gets the label of this EdgeRecordInstance.  # noqa: E501


        :return: The label of this EdgeRecordInstance.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this EdgeRecordInstance.


        :param label: The label of this EdgeRecordInstance.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and label is None:  # noqa: E501
            raise ValueError("Invalid value for `label`, must not be `None`")  # noqa: E501

        self._label = label

    @property
    def properties(self):
        """Gets the properties of this EdgeRecordInstance.  # noqa: E501


        :return: The properties of this EdgeRecordInstance.  # noqa: E501
        :rtype: object
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this EdgeRecordInstance.


        :param properties: The properties of this EdgeRecordInstance.  # noqa: E501
        :type: object
        """
        if self.local_vars_configuration.client_side_validation and properties is None:  # noqa: E501
            raise ValueError("Invalid value for `properties`, must not be `None`")  # noqa: E501

        self._properties = properties

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EdgeRecordInstance):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, EdgeRecordInstance):
            return True

        return self.to_dict() != other.to_dict()



from abc import ABC, abstractmethod
from typing import List

from kag.solver.common.base import KagBaseModule


class ChunkRetrieverABC(KagBaseModule, ABC):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
    """
    An abstract base class for chunk retrieval strategies.

    This class provides a template for implementing different retrieval and reranking strategies for chunks of text.

    Methods:
        recall_docs(query: str, top_k: int = 5, **kwargs) -> List[str]:
            Recalls documents based on the given query.

        rerank_docs(queries: List[str], passages: List[str]) -> List[str]:
            Reranks the retrieved passages based on the given queries.
    """

    @abstractmethod
    def recall_docs(self, query: str, top_k: int = 5, **kwargs) -> List[str]:
        """
        Recalls documents based on the given query.

        Parameters:
            query (str): The query string to search for.
            top_k (int, optional): The number of top documents to return. Defaults to 5.
            **kwargs: Additional keyword arguments for retrieval.

        Returns:
            List[str]: A list of recalled document IDs or content.
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    def rerank_docs(self, queries: List[str], passages: List[str]) -> List[str]:
        """
        Reranks the retrieved passages based on the given queries.

        Parameters:
            queries (List[str]): A list of query strings.
            passages (List[str]): A list of retrieved passages.

        Returns:
            List[str]: A list of reranked passage IDs or content.
        """
        raise NotImplementedError("Subclasses must implement this method")



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.


import numpy as np
from typing import List

from .reranker import Reranker


def rrf_score(length, r: int = 1):
    """
    Calculates the RRF (Recursive Robust Function) scores.
    
    This function generates a score sequence of the given length, where each score is calculated based on the index according to the formula 1/(r+i).
    RRF is a method used in information retrieval and data analysis, and this function provides a way to generate weights based on document indices.
    
    Parameters:
    length: int, the length of the score sequence, i.e., the number of scores to generate.
    r: int, optional, default is 1. Controls the starting index of the scores. Increasing the value of r shifts the emphasis towards later scores.
    
    Returns:
    numpy.ndarray, an array containing the scores calculated according to the given formula.
    """
    return np.array([1 / (r + i) for i in range(length)])



class BGEReranker(Reranker):
    """
    BGEReranker class is a subclass of Reranker that reranks given queries and passages.
    
    This class uses the FlagReranker model from FlagEmbedding to score and reorder passages.
    
    Args:
        model_path (str): Path to the FlagReranker model.
        use_fp16 (bool): Whether to use half-precision floating-point numbers for computation. Default is True.
    """
    def __init__(self, model_path: str, use_fp16: bool = True):
        from FlagEmbedding import FlagReranker
        self.model_path = model_path
        self.model = FlagReranker(self.model_path, use_fp16=use_fp16)

    def rerank(self, queries: List[str], passages: List[str]):
        """
        Reranks given queries and passages.
        
        Args:
            queries (List[str]): List of queries.
            passages (List[str]): List of passages, where each passage is a string.
            
        Returns:
            new_passages (List[str]): List of passages after reranking.
        """
        # Calculate initial ranking scores for passages
        rank_scores = rrf_score(len(passages))
        passage_scores = np.zeros(len(passages)) + rank_scores
        
        # For each query, compute passage scores using the model and accumulate them
        for query in queries:
            scores = self.model.compute_score([[query, x] for x in passages])
            sorted_idx = np.argsort(-np.array(scores))
            for rank, passage_id in enumerate(sorted_idx):
                passage_scores[passage_id] += rank_scores[rank]
        
        # Perform final sorting of passages based on accumulated scores
        merged_sorted_idx = np.argsort(-passage_scores)
        
        new_passages = [passages[x] for x in merged_sorted_idx]
        return new_passages


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: arks.proto

# maya dns filter方式使用的配置文件
# 因为使用的pb协议，所以需要使用pb.proto生产arks_pb2.py代码
# 参考文档: https://yuque.antfin-inc.com/ai-infra/ndhopc/smk38dcs9zqr1ssh#Kb7e0

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='arks.proto',
  package='arks',
  syntax='proto2',
  serialized_options=_b('\n\025com.alipay.arks.proto'),
  serialized_pb=_b('\n\narks.proto\x12\x04\x61rks\"\xfc\x01\n\x13InferTensorContents\x12\x14\n\x0cstring_value\x18\x01 \x03(\t\x12\x12\n\nbool_value\x18\x02 \x03(\x08\x12\x11\n\tint_value\x18\x03 \x03(\x05\x12\x13\n\x0bint64_value\x18\x04 \x03(\x03\x12\x12\n\nuint_value\x18\x05 \x03(\r\x12\x14\n\x0cuint64_value\x18\x06 \x03(\x04\x12\x12\n\nfp32_value\x18\x07 \x03(\x02\x12\x12\n\nfp64_value\x18\x08 \x03(\x01\x12\x12\n\nbyte_value\x18\t \x03(\x0c\x12-\n\x04type\x18\n \x01(\x0e\x32\x11.arks.ContentType:\x0cTYPE_INVALID\"q\n\x04Pair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12+\n\x08\x63ontents\x18\x03 \x01(\x0b\x32\x19.arks.InferTensorContents\x12\x10\n\x08pb_value\x18\x04 \x03(\x0c\x12\x0e\n\x06shapes\x18\x05 \x03(\x05\"\x97\x01\n\x06RowKey\x12\x0f\n\x07row_key\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\x03\x12\x1a\n\x12\x61nt_fea_track_info\x18\x03 \x01(\t\x12\'\n\npartitions\x18\x04 \x03(\x0b\x32\x13.arks.PartitionInfo\x12%\n\x11realtime_features\x18\x05 \x03(\x0b\x32\n.arks.Pair\",\n\rPartitionInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xb9\x01\n\x04Item\x12\x0f\n\x07item_id\x18\x01 \x02(\t\x12\x1c\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32\n.arks.Pair\x12\x1e\n\nattributes\x18\x03 \x03(\x0b\x32\n.arks.Pair\x12\r\n\x05score\x18\x04 \x01(\x02\x12 \n\tsub_items\x18\x05 \x03(\x0b\x32\r.arks.SubItem\x12\x1d\n\x11is_features_valid\x18\x06 \x03(\x08\x42\x02\x10\x01\x12\x12\n\x06scores\x18\x07 \x03(\x02\x42\x02\x10\x01\"\x9a\x01\n\x07SubItem\x12\x0f\n\x07item_id\x18\x01 \x01(\t\x12\x1c\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32\n.arks.Pair\x12\r\n\x05score\x18\x03 \x01(\x02\x12\x1d\n\x11is_features_valid\x18\x04 \x03(\x08\x42\x02\x10\x01\x12\x12\n\x06scores\x18\x05 \x03(\x02\x42\x02\x10\x01\x12\x1e\n\nattributes\x18\x06 \x03(\x0b\x32\n.arks.Pair\"\xc7\x03\n\x08SeekPlan\x12\x14\n\x0cstorage_type\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\x12\x15\n\rcolumn_family\x18\x03 \x01(\t\x12\x0f\n\x07\x63olumns\x18\x04 \x03(\t\x12\x12\n\nkvpair_sep\x18\x05 \x01(\t\x12\x0e\n\x06kv_sep\x18\x06 \x01(\t\x12\x0f\n\x07\x63luster\x18\x07 \x01(\t\x12\x1e\n\x08row_keys\x18\x08 \x03(\x0b\x32\x0c.arks.RowKey\x12\x12\n\ntimeout_ms\x18\t \x01(\x05\x12\x1b\n\x13\x63\x61\x63he_expire_second\x18\n \x01(\x05\x12\x10\n\x08url_user\x18\x0b \x01(\t\x12\x10\n\x08url_item\x18\x0c \x01(\t\x12\x17\n\x0f\x61nt_feature_req\x18\r \x01(\x0c\x12\n\n\x02id\x18\x0e \x01(\t\x12\x16\n\x0ekb_feature_req\x18\x0f \x01(\x0c\x12\x11\n\tdebuginfo\x18\x10 \x01(\t\x12\x11\n\tseparator\x18\x11 \x01(\t\x12=\n\x12item_sequence_type\x18\x12 \x01(\x0e\x32\x16.arks.ItemSequenceType:\tTYPE_NONE\x12\"\n\x0emissing_values\x18\x13 \x03(\x0b\x32\n.arks.Pair\"\x8f\x01\n\x0b\x44umpReqInfo\x12\x0e\n\x06time_s\x18\x01 \x01(\x05\x12\x0e\n\x06oss_id\x18\x02 \x01(\t\x12\x0f\n\x07oss_key\x18\x03 \x01(\t\x12\x13\n\x0btarget_addr\x18\x04 \x01(\t\x12\x10\n\x08query_id\x18\x05 \x01(\x03\x12\r\n\x05token\x18\x06 \x01(\t\x12\x0b\n\x03\x61pp\x18\x07 \x01(\t\x12\x0c\n\x04host\x18\x08 \x01(\t\"\xb3\x04\n\x0b\x41rksRequest\x12\x12\n\x07version\x18\x01 \x01(\x05:\x01\x31\x12\r\n\x05\x64\x65\x62ug\x18\x02 \x01(\x05\x12\x0f\n\x07is_ping\x18\x03 \x01(\x08\x12\x12\n\nsession_id\x18\x04 \x01(\t\x12\x13\n\x0b\x62ucket_name\x18\x05 \x01(\t\x12\x0b\n\x03uid\x18\x06 \x01(\t\x12 \n\x0cuser_profile\x18\x07 \x03(\x0b\x32\n.arks.Pair\x12\"\n\x0escene_features\x18\x08 \x03(\x0b\x32\n.arks.Pair\x12\x19\n\x05items\x18\t \x03(\x0b\x32\n.arks.Item\x12\x15\n\x07is_sort\x18\n \x01(\x08:\x04true\x12\x11\n\x05\x63ount\x18\x0b \x01(\x05:\x02\x31\x30\x12.\n\nout_format\x18\x0c \x01(\x0e\x32\x16.arks.OutputFormatType:\x02PB\x12\x12\n\nchain_name\x18\r \x01(\t\x12\x0b\n\x03scm\x18\x0e \x01(\t\x12\x12\n\nscene_name\x18\x0f \x01(\t\x12\x14\n\x0citem_schemas\x18\x10 \x03(\t\x12\x18\n\x10sub_item_schemas\x18\x11 \x03(\t\x12\"\n\nseek_plans\x18\x12 \x03(\x0b\x32\x0e.arks.SeekPlan\x12(\n\rdump_req_info\x18\x13 \x01(\x0b\x32\x11.arks.DumpReqInfo\x12\x10\n\x08\x61pp_name\x18\x14 \x01(\t\x12\x16\n\x0ereq_timeout_ms\x18\x15 \x01(\x04\x12\x16\n\x0e\x63lient_version\x18\x16 \x01(\t\x12\n\n\x02ip\x18\x17 \x01(\t\"\xba\x02\n\x0c\x41rksResponse\x12,\n\nerror_code\x18\x01 \x01(\x0e\x32\x0f.arks.ErrorCode:\x07SUCCESS\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x13\n\x0b\x62ucket_name\x18\x03 \x01(\t\x12 \n\x0cuser_profile\x18\x04 \x03(\x0b\x32\n.arks.Pair\x12\x19\n\x05items\x18\x05 \x03(\x0b\x32\n.arks.Item\x12\x11\n\tdebug_msg\x18\x06 \x01(\t\x12\x0b\n\x03scm\x18\x07 \x01(\t\x12\"\n\nseek_plans\x18\x08 \x03(\x0b\x32\x0e.arks.SeekPlan\x12\x0f\n\x07\x65rr_msg\x18\t \x01(\t\x12\x10\n\x08\x61lgo_ret\x18\n \x01(\x05\x12\x10\n\x08\x61lgo_msg\x18\x0b \x01(\t\x12\x11\n\ttrace_msg\x18\x0c \x01(\t\x12\n\n\x02rt\x18\r \x01(\x05*T\n\x10OutputFormatType\x12\x06\n\x02PB\x10\x01\x12\x08\n\x04JSON\x10\x02\x12\x08\n\x04TEXT\x10\x03\x12\r\n\tSNAPPY_PB\x10\x04\x12\x06\n\x02\x46\x42\x10\x05\x12\r\n\tSNAPPY_FB\x10\x06*\x86\x01\n\tErrorCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07TIMEOUT\x10\x01\x12\r\n\tSCENE_ERR\x10\x02\x12\r\n\tPARAM_ERR\x10\x03\x12\x0e\n\nSYSTEM_ERR\x10\x04\x12\x0f\n\x0bSERVICE_ERR\x10\x05\x12\x10\n\x0c\x46LOW_CONTROL\x10\x06\x12\x0e\n\nOTHERS_ERR\x10\x07*\xae\x01\n\x0b\x43ontentType\x12\x10\n\x0cTYPE_INVALID\x10\x00\x12\r\n\tTYPE_BOOL\x10\x01\x12\x0e\n\nTYPE_INT32\x10\x02\x12\x0e\n\nTYPE_INT64\x10\x03\x12\x0f\n\x0bTYPE_UINT32\x10\x04\x12\x0f\n\x0bTYPE_UINT64\x10\x05\x12\r\n\tTYPE_FP32\x10\x06\x12\r\n\tTYPE_FP64\x10\x07\x12\x0f\n\x0bTYPE_STRING\x10\x08\x12\r\n\tTYPE_BYTE\x10\t*A\n\x10ItemSequenceType\x12\r\n\tTYPE_NONE\x10\x00\x12\x0f\n\x0bTYPE_CONCAT\x10\x01\x12\r\n\tTYPE_FLAT\x10\x02\x42\x17\n\x15\x63om.alipay.arks.proto')
)

_OUTPUTFORMATTYPE = _descriptor.EnumDescriptor(
  name='OutputFormatType',
  full_name='arks.OutputFormatType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PB', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JSON', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEXT', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SNAPPY_PB', index=3, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FB', index=4, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SNAPPY_FB', index=5, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2422,
  serialized_end=2506,
)
_sym_db.RegisterEnumDescriptor(_OUTPUTFORMATTYPE)

OutputFormatType = enum_type_wrapper.EnumTypeWrapper(_OUTPUTFORMATTYPE)
_ERRORCODE = _descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='arks.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIMEOUT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCENE_ERR', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARAM_ERR', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SYSTEM_ERR', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVICE_ERR', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FLOW_CONTROL', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHERS_ERR', index=7, number=7,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2509,
  serialized_end=2643,
)
_sym_db.RegisterEnumDescriptor(_ERRORCODE)

ErrorCode = enum_type_wrapper.EnumTypeWrapper(_ERRORCODE)
_CONTENTTYPE = _descriptor.EnumDescriptor(
  name='ContentType',
  full_name='arks.ContentType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TYPE_INVALID', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_BOOL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_INT32', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_INT64', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_UINT32', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_UINT64', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_FP32', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_FP64', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_STRING', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_BYTE', index=9, number=9,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2646,
  serialized_end=2820,
)
_sym_db.RegisterEnumDescriptor(_CONTENTTYPE)

ContentType = enum_type_wrapper.EnumTypeWrapper(_CONTENTTYPE)
_ITEMSEQUENCETYPE = _descriptor.EnumDescriptor(
  name='ItemSequenceType',
  full_name='arks.ItemSequenceType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TYPE_NONE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_CONCAT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TYPE_FLAT', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2822,
  serialized_end=2887,
)
_sym_db.RegisterEnumDescriptor(_ITEMSEQUENCETYPE)

ItemSequenceType = enum_type_wrapper.EnumTypeWrapper(_ITEMSEQUENCETYPE)
PB = 1
JSON = 2
TEXT = 3
SNAPPY_PB = 4
FB = 5
SNAPPY_FB = 6
SUCCESS = 0
TIMEOUT = 1
SCENE_ERR = 2
PARAM_ERR = 3
SYSTEM_ERR = 4
SERVICE_ERR = 5
FLOW_CONTROL = 6
OTHERS_ERR = 7
TYPE_INVALID = 0
TYPE_BOOL = 1
TYPE_INT32 = 2
TYPE_INT64 = 3
TYPE_UINT32 = 4
TYPE_UINT64 = 5
TYPE_FP32 = 6
TYPE_FP64 = 7
TYPE_STRING = 8
TYPE_BYTE = 9
TYPE_NONE = 0
TYPE_CONCAT = 1
TYPE_FLAT = 2



_INFERTENSORCONTENTS = _descriptor.Descriptor(
  name='InferTensorContents',
  full_name='arks.InferTensorContents',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_value', full_name='arks.InferTensorContents.string_value', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='arks.InferTensorContents.bool_value', index=1,
      number=2, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int_value', full_name='arks.InferTensorContents.int_value', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64_value', full_name='arks.InferTensorContents.int64_value', index=3,
      number=4, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint_value', full_name='arks.InferTensorContents.uint_value', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint64_value', full_name='arks.InferTensorContents.uint64_value', index=5,
      number=6, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fp32_value', full_name='arks.InferTensorContents.fp32_value', index=6,
      number=7, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fp64_value', full_name='arks.InferTensorContents.fp64_value', index=7,
      number=8, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='byte_value', full_name='arks.InferTensorContents.byte_value', index=8,
      number=9, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='arks.InferTensorContents.type', index=9,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21,
  serialized_end=273,
)


_PAIR = _descriptor.Descriptor(
  name='Pair',
  full_name='arks.Pair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='arks.Pair.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='arks.Pair.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contents', full_name='arks.Pair.contents', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pb_value', full_name='arks.Pair.pb_value', index=3,
      number=4, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shapes', full_name='arks.Pair.shapes', index=4,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=275,
  serialized_end=388,
)


_ROWKEY = _descriptor.Descriptor(
  name='RowKey',
  full_name='arks.RowKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='row_key', full_name='arks.RowKey.row_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='versions', full_name='arks.RowKey.versions', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ant_fea_track_info', full_name='arks.RowKey.ant_fea_track_info', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='partitions', full_name='arks.RowKey.partitions', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='realtime_features', full_name='arks.RowKey.realtime_features', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=391,
  serialized_end=542,
)


_PARTITIONINFO = _descriptor.Descriptor(
  name='PartitionInfo',
  full_name='arks.PartitionInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='arks.PartitionInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='arks.PartitionInfo.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=544,
  serialized_end=588,
)


_ITEM = _descriptor.Descriptor(
  name='Item',
  full_name='arks.Item',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='item_id', full_name='arks.Item.item_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='features', full_name='arks.Item.features', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='arks.Item.attributes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='arks.Item.score', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sub_items', full_name='arks.Item.sub_items', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_features_valid', full_name='arks.Item.is_features_valid', index=5,
      number=6, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scores', full_name='arks.Item.scores', index=6,
      number=7, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=591,
  serialized_end=776,
)


_SUBITEM = _descriptor.Descriptor(
  name='SubItem',
  full_name='arks.SubItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='item_id', full_name='arks.SubItem.item_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='features', full_name='arks.SubItem.features', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='arks.SubItem.score', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_features_valid', full_name='arks.SubItem.is_features_valid', index=3,
      number=4, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scores', full_name='arks.SubItem.scores', index=4,
      number=5, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='arks.SubItem.attributes', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=779,
  serialized_end=933,
)


_SEEKPLAN = _descriptor.Descriptor(
  name='SeekPlan',
  full_name='arks.SeekPlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storage_type', full_name='arks.SeekPlan.storage_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='table', full_name='arks.SeekPlan.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='column_family', full_name='arks.SeekPlan.column_family', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='columns', full_name='arks.SeekPlan.columns', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kvpair_sep', full_name='arks.SeekPlan.kvpair_sep', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kv_sep', full_name='arks.SeekPlan.kv_sep', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cluster', full_name='arks.SeekPlan.cluster', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='row_keys', full_name='arks.SeekPlan.row_keys', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeout_ms', full_name='arks.SeekPlan.timeout_ms', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cache_expire_second', full_name='arks.SeekPlan.cache_expire_second', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='url_user', full_name='arks.SeekPlan.url_user', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='url_item', full_name='arks.SeekPlan.url_item', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ant_feature_req', full_name='arks.SeekPlan.ant_feature_req', index=12,
      number=13, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='arks.SeekPlan.id', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kb_feature_req', full_name='arks.SeekPlan.kb_feature_req', index=14,
      number=15, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='debuginfo', full_name='arks.SeekPlan.debuginfo', index=15,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='separator', full_name='arks.SeekPlan.separator', index=16,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='item_sequence_type', full_name='arks.SeekPlan.item_sequence_type', index=17,
      number=18, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='missing_values', full_name='arks.SeekPlan.missing_values', index=18,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=936,
  serialized_end=1391,
)


_DUMPREQINFO = _descriptor.Descriptor(
  name='DumpReqInfo',
  full_name='arks.DumpReqInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_s', full_name='arks.DumpReqInfo.time_s', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oss_id', full_name='arks.DumpReqInfo.oss_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oss_key', full_name='arks.DumpReqInfo.oss_key', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='target_addr', full_name='arks.DumpReqInfo.target_addr', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query_id', full_name='arks.DumpReqInfo.query_id', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='arks.DumpReqInfo.token', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='app', full_name='arks.DumpReqInfo.app', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host', full_name='arks.DumpReqInfo.host', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1394,
  serialized_end=1537,
)


_ARKSREQUEST = _descriptor.Descriptor(
  name='ArksRequest',
  full_name='arks.ArksRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='arks.ArksRequest.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='debug', full_name='arks.ArksRequest.debug', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_ping', full_name='arks.ArksRequest.is_ping', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='session_id', full_name='arks.ArksRequest.session_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bucket_name', full_name='arks.ArksRequest.bucket_name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='arks.ArksRequest.uid', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_profile', full_name='arks.ArksRequest.user_profile', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scene_features', full_name='arks.ArksRequest.scene_features', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='items', full_name='arks.ArksRequest.items', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_sort', full_name='arks.ArksRequest.is_sort', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='arks.ArksRequest.count', index=10,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=10,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='out_format', full_name='arks.ArksRequest.out_format', index=11,
      number=12, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chain_name', full_name='arks.ArksRequest.chain_name', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scm', full_name='arks.ArksRequest.scm', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scene_name', full_name='arks.ArksRequest.scene_name', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='item_schemas', full_name='arks.ArksRequest.item_schemas', index=15,
      number=16, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sub_item_schemas', full_name='arks.ArksRequest.sub_item_schemas', index=16,
      number=17, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seek_plans', full_name='arks.ArksRequest.seek_plans', index=17,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dump_req_info', full_name='arks.ArksRequest.dump_req_info', index=18,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='app_name', full_name='arks.ArksRequest.app_name', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='req_timeout_ms', full_name='arks.ArksRequest.req_timeout_ms', index=20,
      number=21, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='client_version', full_name='arks.ArksRequest.client_version', index=21,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ip', full_name='arks.ArksRequest.ip', index=22,
      number=23, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1540,
  serialized_end=2103,
)


_ARKSRESPONSE = _descriptor.Descriptor(
  name='ArksResponse',
  full_name='arks.ArksResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='arks.ArksResponse.error_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='session_id', full_name='arks.ArksResponse.session_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bucket_name', full_name='arks.ArksResponse.bucket_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_profile', full_name='arks.ArksResponse.user_profile', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='items', full_name='arks.ArksResponse.items', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='debug_msg', full_name='arks.ArksResponse.debug_msg', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scm', full_name='arks.ArksResponse.scm', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seek_plans', full_name='arks.ArksResponse.seek_plans', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='err_msg', full_name='arks.ArksResponse.err_msg', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='algo_ret', full_name='arks.ArksResponse.algo_ret', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='algo_msg', full_name='arks.ArksResponse.algo_msg', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trace_msg', full_name='arks.ArksResponse.trace_msg', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rt', full_name='arks.ArksResponse.rt', index=12,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2106,
  serialized_end=2420,
)

_INFERTENSORCONTENTS.fields_by_name['type'].enum_type = _CONTENTTYPE
_PAIR.fields_by_name['contents'].message_type = _INFERTENSORCONTENTS
_ROWKEY.fields_by_name['partitions'].message_type = _PARTITIONINFO
_ROWKEY.fields_by_name['realtime_features'].message_type = _PAIR
_ITEM.fields_by_name['features'].message_type = _PAIR
_ITEM.fields_by_name['attributes'].message_type = _PAIR
_ITEM.fields_by_name['sub_items'].message_type = _SUBITEM
_SUBITEM.fields_by_name['features'].message_type = _PAIR
_SUBITEM.fields_by_name['attributes'].message_type = _PAIR
_SEEKPLAN.fields_by_name['row_keys'].message_type = _ROWKEY
_SEEKPLAN.fields_by_name['item_sequence_type'].enum_type = _ITEMSEQUENCETYPE
_SEEKPLAN.fields_by_name['missing_values'].message_type = _PAIR
_ARKSREQUEST.fields_by_name['user_profile'].message_type = _PAIR
_ARKSREQUEST.fields_by_name['scene_features'].message_type = _PAIR
_ARKSREQUEST.fields_by_name['items'].message_type = _ITEM
_ARKSREQUEST.fields_by_name['out_format'].enum_type = _OUTPUTFORMATTYPE
_ARKSREQUEST.fields_by_name['seek_plans'].message_type = _SEEKPLAN
_ARKSREQUEST.fields_by_name['dump_req_info'].message_type = _DUMPREQINFO
_ARKSRESPONSE.fields_by_name['error_code'].enum_type = _ERRORCODE
_ARKSRESPONSE.fields_by_name['user_profile'].message_type = _PAIR
_ARKSRESPONSE.fields_by_name['items'].message_type = _ITEM
_ARKSRESPONSE.fields_by_name['seek_plans'].message_type = _SEEKPLAN
DESCRIPTOR.message_types_by_name['InferTensorContents'] = _INFERTENSORCONTENTS
DESCRIPTOR.message_types_by_name['Pair'] = _PAIR
DESCRIPTOR.message_types_by_name['RowKey'] = _ROWKEY
DESCRIPTOR.message_types_by_name['PartitionInfo'] = _PARTITIONINFO
DESCRIPTOR.message_types_by_name['Item'] = _ITEM
DESCRIPTOR.message_types_by_name['SubItem'] = _SUBITEM
DESCRIPTOR.message_types_by_name['SeekPlan'] = _SEEKPLAN
DESCRIPTOR.message_types_by_name['DumpReqInfo'] = _DUMPREQINFO
DESCRIPTOR.message_types_by_name['ArksRequest'] = _ARKSREQUEST
DESCRIPTOR.message_types_by_name['ArksResponse'] = _ARKSRESPONSE
DESCRIPTOR.enum_types_by_name['OutputFormatType'] = _OUTPUTFORMATTYPE
DESCRIPTOR.enum_types_by_name['ErrorCode'] = _ERRORCODE
DESCRIPTOR.enum_types_by_name['ContentType'] = _CONTENTTYPE
DESCRIPTOR.enum_types_by_name['ItemSequenceType'] = _ITEMSEQUENCETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InferTensorContents = _reflection.GeneratedProtocolMessageType('InferTensorContents', (_message.Message,), dict(
  DESCRIPTOR = _INFERTENSORCONTENTS,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.InferTensorContents)
  ))
_sym_db.RegisterMessage(InferTensorContents)

Pair = _reflection.GeneratedProtocolMessageType('Pair', (_message.Message,), dict(
  DESCRIPTOR = _PAIR,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.Pair)
  ))
_sym_db.RegisterMessage(Pair)

RowKey = _reflection.GeneratedProtocolMessageType('RowKey', (_message.Message,), dict(
  DESCRIPTOR = _ROWKEY,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.RowKey)
  ))
_sym_db.RegisterMessage(RowKey)

PartitionInfo = _reflection.GeneratedProtocolMessageType('PartitionInfo', (_message.Message,), dict(
  DESCRIPTOR = _PARTITIONINFO,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.PartitionInfo)
  ))
_sym_db.RegisterMessage(PartitionInfo)

Item = _reflection.GeneratedProtocolMessageType('Item', (_message.Message,), dict(
  DESCRIPTOR = _ITEM,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.Item)
  ))
_sym_db.RegisterMessage(Item)

SubItem = _reflection.GeneratedProtocolMessageType('SubItem', (_message.Message,), dict(
  DESCRIPTOR = _SUBITEM,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.SubItem)
  ))
_sym_db.RegisterMessage(SubItem)

SeekPlan = _reflection.GeneratedProtocolMessageType('SeekPlan', (_message.Message,), dict(
  DESCRIPTOR = _SEEKPLAN,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.SeekPlan)
  ))
_sym_db.RegisterMessage(SeekPlan)

DumpReqInfo = _reflection.GeneratedProtocolMessageType('DumpReqInfo', (_message.Message,), dict(
  DESCRIPTOR = _DUMPREQINFO,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.DumpReqInfo)
  ))
_sym_db.RegisterMessage(DumpReqInfo)

ArksRequest = _reflection.GeneratedProtocolMessageType('ArksRequest', (_message.Message,), dict(
  DESCRIPTOR = _ARKSREQUEST,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.ArksRequest)
  ))
_sym_db.RegisterMessage(ArksRequest)

ArksResponse = _reflection.GeneratedProtocolMessageType('ArksResponse', (_message.Message,), dict(
  DESCRIPTOR = _ARKSRESPONSE,
  __module__ = 'arks_pb2'
  # @@protoc_insertion_point(class_scope:arks.ArksResponse)
  ))
_sym_db.RegisterMessage(ArksResponse)


DESCRIPTOR._options = None
_ITEM.fields_by_name['is_features_valid']._options = None
_ITEM.fields_by_name['scores']._options = None
_SUBITEM.fields_by_name['is_features_valid']._options = None
_SUBITEM.fields_by_name['scores']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

from typing import Type, List, Union
import re
import os

from kag.builder.model.chunk import Chunk, ChunkTypeEnum
from kag.interface.builder.splitter_abc import SplitterABC
from knext.common.base.runnable import Input, Output


class PatternSplitter(SplitterABC):
    def __init__(self, pattern_dict: dict = None, chunk_cut_num=None):
        """
        pattern_dict:
        {
            "pattern": 匹配pattern,
            "group": {
                "header":1,
                "name":2,
                "content":3
            }
        }
        """
        super().__init__()
        if pattern_dict is None:
            pattern_dict = {
                "pattern": r"(\d+)\.([^0-9]+?)？([^0-9第版].*?)(?=\d+\.|$)",
                "group": {"header": 2, "name": 2, "content": 0},
            }
        self.pattern = pattern_dict["pattern"]
        self.group = pattern_dict["group"]
        self.chunk_cut_num = chunk_cut_num

    @property
    def input_types(self) -> Type[Input]:
        """The type of input this Runnable object accepts specified as a type annotation."""
        return Chunk

    @property
    def output_types(self) -> Type[Output]:
        """The type of output this Runnable object produces specified as a type annotation."""
        return List[Chunk]

    def split_sentence(self, content):
        sentence_delimiters = "。？?！!；;\n"
        output = []
        start = 0
        for idx, char in enumerate(content):
            if char in sentence_delimiters:
                end = idx
                tmp = content[start : end + 1].strip()
                if len(tmp) > 0:
                    output.append(tmp)
                start = idx + 1
        res = content[start:]
        if len(res) > 0:
            output.append(res)
        return output

    def slide_window_chunk(
        self,
        content: Union[str, List[str]],
        chunk_size: int = 2000,
        window_length: int = 300,
        sep: str = "\n",
        prefix: str = "SlideWindow",
    ) -> List[Chunk]:

        if isinstance(content, str):
            content = self.split_sentence(content)
        splitted = []
        cur = []
        cur_len = 0
        for sentence in content:
            if cur_len + len(sentence) > chunk_size:
                splitted.append(cur)
                tmp = []
                cur_len = 0
                for item in cur[::-1]:
                    if cur_len >= window_length:
                        break
                    tmp.append(item)
                    cur_len += len(item)
                cur = tmp[::-1]

            cur.append(sentence)
            cur_len += len(sentence)
        if len(cur) > 0:
            splitted.append(cur)

        output = []
        for idx, sentences in enumerate(splitted):
            chunk_name = f"{prefix}#{idx}"
            chunk = Chunk(
                id=Chunk.generate_hash_id(chunk_name),
                name=chunk_name,
                content=sep.join(sentences),
            )
            output.append(chunk)
        return output

    def chunk_split(
        self,
        chunk: Chunk,
    ) -> List[Chunk]:
        text = chunk.content

        pattern = re.compile(self.pattern, re.DOTALL)

        # 查找所有匹配项
        matches = pattern.finditer(text)

        # 遍历所有匹配项
        chunks = []
        for match in matches:
            chunk = Chunk(
                chunk_header=match.group(self.group["header"]),
                name=match.group(self.group["name"]),
                id=Chunk.generate_hash_id(match.group(self.group["content"])),
                content=match.group(self.group["content"]),
            )
            chunk = [chunk]

            if self.chunk_cut_num:
                chunk = self.slide_window_chunk(
                    content=chunk[0].content,
                    chunk_size=self.chunk_cut_num,
                    window_length=self.chunk_cut_num / 4,
                    sep="\n",
                    prefix=chunk[0].name,
                )

            chunks.extend(chunk)

        return chunks

    def invoke(self, input: Chunk, **kwargs) -> List[Output]:

        chunks = self.chunk_split(input)
        return chunks

    def to_rest(self):
        pass

    @classmethod
    def from_rest(cls, rest_model):
        pass


class LayeredPatternSpliter(PatternSplitter):
    pass


def _test():
    pattern_dict = {
        "pattern": r"(\d+)\.([^0-9]+?)？([^0-9第版].*?)(?=\d+\.|$)",
        "group": {"header": 2, "name": 2, "content": 0},
    }
    ds = PatternSplitter(pattern_dict=pattern_dict)
    from kag.builder.component.reader.pdf_reader import PDFReader

    reader = PDFReader()
    file_path = os.path.dirname(__file__)
    test_file_path = os.path.join(file_path, "../../../../tests/builder/data/aiwen.pdf")
    pre_output = reader._handle(test_file_path)

    handle_input = pre_output[0]
    handle_result = ds._handle(handle_input)
    print("handle_result", handle_result)

    return handle_result


if __name__ == "__main__":
    res = _test()
    print(res)



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

from kag.interface.builder.reader_abc import SourceReaderABC
from kag.interface.builder.splitter_abc import SplitterABC
from kag.interface.builder.extractor_abc import ExtractorABC
from kag.interface.builder.mapping_abc import MappingABC
from kag.interface.builder.aligner_abc import AlignerABC
from kag.interface.builder.writer_abc import SinkWriterABC
from knext.builder.builder_chain_abc import BuilderChainABC

__all__ = [
    "SourceReaderABC",
    "SplitterABC",
    "ExtractorABC",
    "MappingABC",
    "AlignerABC",
    "SinkWriterABC",
    "BuilderChainABC"
]



import logging

from kag.solver.prompt.default.logic_form_plan import LogicFormPlanPrompt

logger = logging.getLogger(__name__)



class LawLogicFormPlanPrompt(LogicFormPlanPrompt):
    default_case_zh = """"cases": [
        {
            "query": "中华人民共和国铁路法第二十八条的内容是什么",
            "answer": "Step1:中华人民共和国铁路法第二十八条的内容是什么 ?\nAction1:get_spo(s=s1:Chunk[中华人民共和国铁路法第二十八条], p=p1:content, o=o1:Text)\n Action2: get(o1)"
        }
    ],"""

    template_en = LogicFormPlanPrompt.template_en

    def __init__(self, language: str):
        super().__init__(language)



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

import requests
from typing import Type, List

from kag.builder.component.reader import MarkDownReader
from kag.builder.model.chunk import Chunk
from kag.interface.builder import SourceReaderABC
from knext.common.base.runnable import Input, Output

from kag.common.llm.client import LLMClient


class YuqueReader(SourceReaderABC):
    def __init__(self, token: str, **kwargs):
        super().__init__(**kwargs)
        self.token = token
        self.markdown_reader = MarkDownReader(**kwargs)

    @property
    def input_types(self) -> Type[Input]:
        """The type of input this Runnable object accepts specified as a type annotation."""
        return str

    @property
    def output_types(self) -> Type[Output]:
        """The type of output this Runnable object produces specified as a type annotation."""
        return Chunk

    @staticmethod
    def get_yuque_api_data(token, url):
        headers = {"X-Auth-Token": token}

        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()  # Raise an HTTPError for bad responses (4xx and 5xx)
            return response.json()["data"]  # Assuming the API returns JSON data
        except requests.exceptions.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
        except requests.exceptions.RequestException as err:
            print(f"Error occurred: {err}")
        except Exception as err:
            print(f"An error occurred: {err}")

    def invoke(self, input: str, **kwargs) -> List[Output]:
        if not input:
            raise ValueError("Input cannot be empty")

        url: str = input
        data = self.get_yuque_api_data(self.token, url)
        id = data.get("id", "")
        title = data.get("title", "")
        content = data.get("body", "")

        chunks = self.markdown_reader.solve_content(id, title, content)

        return chunks


import logging
from typing import List

from kag.interface.solver.kag_reasoner_abc import KagReasonerABC
from kag.interface.solver.lf_planner_abc import LFPlannerABC
from kag.solver.implementation.default_kg_retrieval import KGRetrieverByLlm
from kag.solver.implementation.default_lf_planner import DefaultLFPlanner
from kag.solver.implementation.lf_chunk_retriever import LFChunkRetriever
from kag.solver.logic.core_modules.common.base_model import LFPlanResult
from kag.solver.logic.core_modules.lf_solver import LFSolver

logger = logging.getLogger()

class DefaultReasoner(KagReasonerABC):
    """
    A processor class for handling logical form tasks in language processing.

    This class uses an LLM module (llm_module) to plan, retrieve, and solve logical forms.

    Parameters:
    - lf_planner (LFBasePlanner): The planner for structuring logical forms. Defaults to None. If not provided, the default implementation of LFPlanner is used.
    - lf_solver: Instance of the logical form solver, which solves logical form problems. If not provided, the default implementation of LFSolver is used.

    Attributes:
    - lf_planner: Instance of the logical form planner.
    - lf_solver: Instance of the logical form solver, which solves logical form problems.
    - sub_query_total: Total number of sub-queries processed.
    - kg_direct: Number of direct knowledge graph queries.
    - trace_log: List to log trace information.
    """

    def __init__(self, lf_planner: LFPlannerABC = None, lf_solver: LFSolver = None, **kwargs):
        super().__init__(
            lf_planner=lf_planner,
            lf_solver=lf_solver,
            **kwargs
        )

        self.lf_planner = lf_planner or DefaultLFPlanner(**kwargs)
        self.lf_solver = lf_solver or LFSolver(
            kg_retriever=KGRetrieverByLlm(**kwargs),
            chunk_retriever=LFChunkRetriever(**kwargs),
            **kwargs
        )

        self.sub_query_total = 0
        self.kg_direct = 0
        self.trace_log = []

    def reason(self, question: str):
        """
        Processes a given question by planning and executing logical forms to derive an answer.

        Parameters:
        - question (str): The input question to be processed.

        Returns:
        - solved_answer: The final answer derived from solving the logical forms.
        - supporting_fact: Supporting facts gathered during the reasoning process.
        - history_log: A dictionary containing the history of QA pairs and re-ranked documents.
        """
        # logic form planing
        lf_nodes: List[LFPlanResult] = self.lf_planner.lf_planing(question)

        # logic form execution
        solved_answer, sub_qa_pair, recall_docs, history_qa_log = self.lf_solver.solve(question, lf_nodes)
        # Generate supporting facts for sub question-answer pair
        supporting_fact = '\n'.join(sub_qa_pair)

        # Retrieve and rank documents
        sub_querys = [lf.query for lf in lf_nodes]
        if self.lf_solver.chunk_retriever:
            docs = self.lf_solver.chunk_retriever.rerank_docs([question] + sub_querys, recall_docs)
        else:
            logger.info("DefaultReasoner not enable chunk retriever")
            docs = []
        history_log = {
            'history': history_qa_log,
            'rerank_docs': docs
        }
        if len(docs) > 0:
            # Append supporting facts for retrieved chunks
            supporting_fact += f"\nPassages:{str(docs)}"
        return solved_answer, supporting_fact, history_log



import logging
import time
from enum import Enum

from kag.solver.logic.core_modules.common.base_model import Identifer
from kag.solver.logic.core_modules.common.one_hop_graph import KgGraph, EntityData, RelationData
from kag.solver.logic.core_modules.parser.logic_node_parser import FilterNode, ExtractorNode, \
    VerifyNode


class MatchRes(Enum):
    MATCH = 1
    UN_MATCH = 2
    UN_RELATED = 3
    RELATED = 4


class MatchInfo:
    def __init__(self, res: MatchRes, desc: str = ''):
        self.res = res
        self.desc = desc

    def trans_match_res_to_str(self):
        if self.res == MatchRes.RELATED:
            return "相关"
        elif self.res == MatchRes.MATCH:
            return "满足"
        elif self.res == MatchRes.UN_MATCH:
            return "不满足"
        else:
            return "不相关"

def trans_str_res_to_match(res: str):
    if res is None or res == '' or "无相关信息" in res \
            or "不相关" in res:
        return MatchRes.UN_RELATED
    return MatchRes.RELATED


class RuleRunner:
    def __init__(self):
        self.op_map = {
            "equal": self.run_equal,
            "lt": self.run_lt,
            "gt": self.run_gt,
            "le": self.run_le,
            "ge": self.run_ge,
            "in": self.run_in,
            "contains": self.run_contains,
            "and": self.run_and,
            "or": self.run_or,
            "not": self.run_not,
            "match": self.run_match,
            "exist": self.run_exists,
            "necessary": self.run_necessary,
            "collect_in": self.run_collect_in,
            "collect_contains": self.run_collect_contains
        }

    def run_rule(self, op_name: str, left_value, right_value):
        pass

    def run_equal(self, left_value, right_value):
        pass

    def run_gt(self, left_value, right_value):
        pass

    def run_lt(self, left_value, right_value):
        pass

    def run_ge(self, left_value, right_value):
        pass

    def run_le(self, left_value, right_value):
        pass

    def run_in(self, left_value, right_value):
        pass

    def run_contains(self, left_value, right_value):
        pass

    def run_and(self, left_value, right_value):
        pass

    def run_or(self, left_value, right_value):
        pass

    def run_not(self, left_value):
        pass

    def run_match(self, left_value, right_value):
        pass

    def run_exists(self, left_value):
        pass

    def run_necessary(self, left_value):
        pass

    def run_collect_in(self, left_value, right_value):
        pass

    def run_collect_contains(self, left_value, right_value):
        pass


class StrRuleRunner(RuleRunner):
    def __init__(self):
        super().__init__()

    def run_equal(self, left_value, right_value):
        return left_value == right_value

    def run_gt(self, left_value, right_value):
        return str(left_value) > str(right_value)

    def run_lt(self, left_value, right_value):
        return str(left_value) < str(right_value)

    def run_ge(self, left_value, right_value):
        return str(left_value) >= str(right_value)

    def run_le(self, left_value, right_value):
        return str(left_value) <= str(right_value)

    def run_in(self, left_value, right_value):
        return left_value in right_value

    def run_contains(self, left_value, right_value):
        return right_value in left_value

    def run_and(self, left_value, right_value):
        return left_value and right_value

    def run_or(self, left_value, right_value):
        return left_value or right_value

    def run_not(self, left_value):
        return not left_value

    def run_match(self, left_value, right_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_MATCH)
        if not isinstance(left_value, list):
            return MatchInfo(MatchRes.UN_MATCH)
        for v in left_value:
            if v in right_value or right_value in v:
                return MatchInfo(MatchRes.MATCH)
        return MatchInfo(MatchRes.UN_MATCH)

    def run_exists(self, left_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_MATCH)
        if isinstance(left_value, list) and len(left_value) != 0:
            return MatchInfo(MatchRes.MATCH)
        return MatchInfo(MatchRes.UN_MATCH)

    def run_necessary(self, left_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_MATCH)
        if isinstance(left_value, list) and len(left_value) == 1:
            return MatchInfo(MatchRes.MATCH)
        return MatchInfo(MatchRes.UN_MATCH)

    def run_collect_in(self, left_value, right_value):
        pass

    def run_collect_contains(self, left_value, right_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_MATCH)
        if not isinstance(left_value, list):
            return MatchInfo(MatchRes.UN_MATCH)
        for v in left_value:
            if str(right_value) in str(v):
                return MatchInfo(MatchRes.MATCH)
        return MatchInfo(MatchRes.UN_MATCH)


class ModelRunner(StrRuleRunner):
    def __init__(self, llm, kg_graph: KgGraph, query:str, req_id: str):
        super().__init__()
        self.llm = llm
        self.kg_graph = kg_graph
        self.query = query
        self.req_id = req_id
    def _get_kg_graph_data(self):
        return self.kg_graph.to_spo()

    def run_match(self, left_value, right_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        if not isinstance(left_value, list):
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        start_time = time.time()
        prompt = "根据提供的检索文档，请首先判断是否能够直接回答指令“{}”。如果可以直接回答，请直接回复答案，无需解释；如果不能直接回答但存在关联信息，请总结其中与指令“{}”相关的关键信息，并明确解释为何与指令相关；如果没有任何相关信息，直接回复“无相关信息”无需解释。\n【检索文档】：“{}”\n请确保所提供的信息直接准确地来自检索文档，不允许任何自身推测。".format(
            right_value, right_value, str(self._get_kg_graph_data())
        )
        res = self.llm.generate(prompt, max_output_len=100)
        logging.info(f"ModelRunner {self.req_id} cost={time.time() - start_time} prompt={prompt} res={res}")
        return MatchInfo(trans_str_res_to_match(res), res)

    def run_collect_in(self, left_value, right_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        if not isinstance(left_value, list):
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        start_time = time.time()
        prompt = "根据提供的检索文档，请首先判断是否能够直接回答指令“{}”。如果可以直接回答，请直接回复答案，无需解释；如果不能直接回答但存在关联信息，请总结其中与指令“{}”相关的关键信息，并明确解释为何与指令相关；如果没有任何相关信息，直接回复“无相关信息”无需解释。\n【检索文档】：“{}”\n请确保所提供的信息直接准确地来自检索文档，不允许任何自身推测。".format(
            right_value, right_value, str(self._get_kg_graph_data())
        )
        res = self.llm.generate(prompt, max_output_len=100)
        logging.info(f"ModelRunner {self.req_id} cost={time.time() - start_time} prompt={prompt} res={res}")
        return MatchInfo(trans_str_res_to_match(res), res)

    def run_collect_contains(self, left_value, right_value):
        if left_value is None:
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        if not isinstance(left_value, list):
            return MatchInfo(MatchRes.UN_RELATED, "判定信息不足")
        start_time = time.time()
        prompt = "根据提供的检索文档，请首先判断是否能够直接回答指令“{}”。如果可以直接回答，请直接回复答案，无需解释；如果不能直接回答但存在关联信息，请总结其中与指令“{}”相关的关键信息，并明确解释为何与指令相关；如果没有任何相关信息，直接回复“无相关信息”无需解释。\n【检索文档】：“{}”\n请确保所提供的信息直接准确地来自检索文档，不允许任何自身推测。".format(
            right_value, right_value, str(self._get_kg_graph_data())
        )
        res = self.llm.generate(prompt, max_output_len=100)
        logging.info(f"ModelRunner {self.req_id} cost={time.time() - start_time} prompt={prompt} res={res}")
        return MatchInfo(trans_str_res_to_match(res), res)


class OpRunner:
    def __init__(self, kg_graph: KgGraph, llm, query: str, req_id: str):
        self.kg_graph = kg_graph
        self.query = query
        if llm is None:
            self.runner: RuleRunner = StrRuleRunner()
        else:
            self.runner: ModelRunner = ModelRunner(llm, kg_graph, query, req_id)
        self.llm = llm

    def _get_identifer_to_doc(self, alias:Identifer):
        data = self.kg_graph.get_entity_by_alias(alias)
        if data is None:
            return []
        ret_data = []
        for d in data:
            if isinstance(d, EntityData):
                if d.type == "attribute":
                    ret_data.append(d.biz_id)
                else:
                    ret_data.append(d.to_json())
            elif isinstance(d, RelationData):
                ret_data.append(d.to_json())
            else:
                ret_data.append(d)
        return ret_data

    def _get_alias_to_doc(self, alias):
        if not isinstance(alias, list):
            alias_set = [alias]
        else:
            alias_set = set(alias)
        ret_data = []
        for alias_ele in alias_set:
            if isinstance(alias_ele, Identifer):
                ret_data = ret_data + self._get_identifer_to_doc(alias_ele)
            else:
                ret_data.append(alias)
        return ret_data

    def _get_value_ins_identifer(self, alias: Identifer):
        data = self.kg_graph.get_entity_by_alias(alias)
        if data is None:
            return []
        ret_data = []
        for d in data:
            if isinstance(d, EntityData) and d.type == "attribute":
                ret_data.append(d.biz_id)
            else:
                ret_data.append(d)
        return ret_data

    def _get_value_ins(self, alias):
        if isinstance(alias, list):
            alias_set = set(alias)
        else:
            alias_set = {alias}
        ret_data = []
        for alias_ele in alias_set:
            if isinstance(alias_ele, Identifer):
                ret_data = ret_data + self._get_value_ins_identifer(alias_ele)
            else:
                return alias
        return ret_data

    def run_single_binary_exec_rule(self, op_name: str, left_value, right_value):
        left_value = self._get_value_ins(left_value)
        right_value = self._get_value_ins(right_value)
        if left_value is None:
            return {}
        res = {}
        for left_value_ins in left_value:
            # 可能是实体或文本值
            left_value_text = left_value_ins
            if isinstance(left_value_ins, EntityData):
                left_value_text = left_value_ins.biz_id
            single_rule_res = self.runner.op_map[op_name](left_value_text, right_value)
            res[left_value_text] = single_rule_res
        return res

    def run_single_unary_exec_rule(self, op_name: str, left_value):
        left_value = self._get_value_ins(left_value)
        if left_value is None:
            return {}
        res = {}
        for left_value_ins in left_value:
            single_rule_res = self.runner.op_map[op_name](left_value_ins)
            res[left_value_ins] = single_rule_res
        return res

    def single_rule_dispatch(self, op_name: str, left_value, right_value):
        op_name = self._get_op_zh_2_en(op_name)

        binary_op = ['equal', 'lt', 'gt', 'le', 'ge', 'in', 'contains', 'and', 'or']
        unary_op = ['not']

        if op_name in binary_op:
            return self.run_single_binary_exec_rule(op_name, left_value, right_value)
        elif op_name in unary_op:
            return self.run_single_unary_exec_rule(op_name, left_value)
        else:
            raise RuntimeError(f"not impl op {op_name}")

    def collect_rule_dispatch(self, op_name: str, left_value, right_value):
        op_name = self._get_op_zh_2_en(op_name)
        collect_binary_op = ['match', 'contains', 'in']
        collect_unary_op = ['exist', 'necessary']
        if op_name in collect_unary_op:
            return self.run_collect_unary_exec_rule(op_name, left_value)
        elif op_name in collect_binary_op:
            return self.run_collect_binary_exec_rule(op_name, left_value, right_value)
        else:
            # agg by self
            res = self.single_rule_dispatch(op_name, left_value, right_value)
            if res is not None and True in res.values():
                return MatchInfo(MatchRes.MATCH, '')
            return MatchInfo(MatchRes.UN_MATCH, '')

    def run_collect_binary_exec_rule(self, op_name: str, left_value, right_value):
        collect_op_name_map = {
            "in": "collect_in",
            "contains": "collect_contains",
            "necessary": "necessary",
            "match": "match"
        }
        left_value = self._get_value_ins(left_value)
        right_value = self._get_value_ins(right_value)
        """
        res = MatchRes
        """
        res: MatchRes = self.runner.op_map[collect_op_name_map[op_name]](left_value, right_value)
        return res

    def run_collect_unary_exec_rule(self, op_name: str, left_value):
        left_value = self._get_value_ins(left_value)
        """
                res = MatchRes
                """
        res: MatchInfo = self.runner.op_map[op_name](left_value)
        return res

    def _get_op_zh_2_en(self, op_name):
        name_map = {
            "包含": "contains",
            "存在": "exist",
            "匹配": "match",
            "必要": "necessary",
            "等于": "equal",
            "大于": "gt",
            "小于": "lt"
        }
        if op_name not in name_map.keys():
            return op_name
        return name_map[op_name]

    def run_filter_op(self, f: FilterNode):
        # 对边不执行过滤
        if isinstance(f.left_expr, Identifer) and f.left_expr in self.kg_graph.edge_alias:
            return
        res = self.single_rule_dispatch(f.op, f.left_expr, f.right_expr)
        failed_list = []
        for r in res.keys():
            if not res[r]:
                failed_list.append(r)
        self.kg_graph.rmv_ins(f.left_expr, failed_list)

    def run_extractor_op(self, f: ExtractorNode):
        update_verify = VerifyNode("verify", {
            "left_expr": f.alias_set,
            "right_expr": self.query,
            "op": "匹配"
        })
        return self.run_verify_op(update_verify)

    def run_verify_op(self, f: VerifyNode):
        verify_kg_graph = KgGraph()
        left_expr_name = f.get_left_expr_name()
        s_alias_name = f"verify_s_{left_expr_name}"
        p_alias_name = f"verify_p_{left_expr_name}"
        o_alias_name = f"verify_o_{left_expr_name}"
        verify_kg_graph.query_graph[p_alias_name] = {
            "s": s_alias_name,
            "p": p_alias_name,
            "o": o_alias_name
        }
        left_value = self._get_alias_to_doc(f.left_expr)
        if len(left_value) == 0:
            return None, None, None

        verify_kg_graph.nodes_alias.append(s_alias_name)
        verify_kg_graph.nodes_alias.append(p_alias_name)
        verify_kg_graph.edge_alias.append(o_alias_name)
        s_entity_data = EntityData()
        s_entity_data.type = "verify_op"
        s_entity_data.type_zh = "判定"
        s_entity_data.biz_id = f"{left_expr_name}"
        s_entity_data.name = "检索信息"
        s_entity_data.description = "检索信息"
        left_expr_set = f.get_left_expr_set()
        description = []
        for left_epxr in left_expr_set:
            if left_epxr in self.kg_graph.logic_form_base.keys():
                description.append(f"{self.kg_graph.logic_form_base[left_epxr]}")
        if len(description) > 0:
            s_entity_data.description = "\n\n".join(description)
        right_value = f.right_expr
        if right_value is None or right_value == '':
            right_value = self.query
        right_value = self._get_alias_to_doc(right_value)
        match_info = self.collect_rule_dispatch(f.op, f.left_expr, f.right_expr)
        o_entity_data = EntityData()
        o_entity_data.type = "verify_op_result"
        o_entity_data.type_zh = "问题"
        o_entity_data.biz_id = match_info.trans_match_res_to_str()
        o_entity_data.name = match_info.trans_match_res_to_str()
        o_entity_data.description = match_info.desc
        rel = RelationData()
        rel.from_id = s_entity_data.biz_id
        rel.from_type = s_entity_data.type_zh
        rel.end_id = o_entity_data.biz_id
        rel.end_type = o_entity_data.type_zh
        rel.from_entity = s_entity_data
        rel.end_entity = o_entity_data
        rel.type = f"是否{f.op} {right_value}"

        verify_kg_graph.entity_map[s_alias_name] = [s_entity_data]
        verify_kg_graph.edge_map[p_alias_name] = [rel]

        self.kg_graph.merge_kg_graph(verify_kg_graph)
        return match_info, rel, p_alias_name



from abc import ABC
from typing import Union

from kag.solver.common.base import KagBaseModule
from kag.solver.logic.core_modules.common.base_model import LogicNode
from kag.solver.logic.core_modules.common.one_hop_graph import KgGraph
from kag.solver.logic.core_modules.common.schema_utils import SchemaUtils


class OpExecutor(KagBaseModule, ABC):
    """
    Base class for operators execution.

    Each subclass must implement the execution and judgment functions.
    """
    def __init__(self, nl_query: str, kg_graph: KgGraph, schema: SchemaUtils, debug_info: dict, **kwargs):
        """
        Initializes the operator executor with necessary components.

        Parameters:
            nl_query (str): Natural language query string.
            kg_graph (KgGraph): Knowledge graph object for subsequent queries and parsing.
            schema (SchemaUtils): Semantic structure definition to assist in the parsing process.
            debug_info (dict): Debug information dictionary to record debugging information during parsing.
        """
        super().__init__(**kwargs)
        self.kg_graph = kg_graph
        self.schema = schema
        self.nl_query = nl_query
        self.debug_info = debug_info

    def executor(self, logic_node: LogicNode, req_id: str, param: dict) -> Union[KgGraph, list]:
        """
         Executes the operation based on the given logic node.

         This method should be implemented by subclasses to define how the operation is executed.

         Parameters:
             logic_node (LogicNode): The logic node that defines the operation to execute.
             req_id (str): Request identifier.
             param (dict): Parameters needed for the execution.

         Returns:
             Union[KgGraph, list]: The result of the operation, which could be a knowledge graph or a list.
         """
        pass

    def is_this_op(self, logic_node: LogicNode) -> bool:
        """
        Determines if this executor is responsible for the given logic node.

        This method should be implemented by subclasses to specify the conditions under which
        this executor can handle the logic node.

        Parameters:
            logic_node (LogicNode): The logic node to check.

        Returns:
            bool: True if this executor can handle the logic node, False otherwise.
        """
        pass